/*
 * proximity.c
 *
 * SONAR client library v. 0 example program
 *
 * Jason Cox
 *
 * This program takes two arguments, the maximum seconds and maximum msecs
 * allowable on a sonar query.  You can type (or pipe) some host names
 * and the names will be queried after you type EOF.  Don't forget to
 * set your SONAR server host in your environment.
 *
 */

#include "sonar.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

int main(argc, argv)
int argc;
char **argv;
{
    int i, cnt;
    unsigned int iaddr;
    struct sonar_info sonar[185], sonar_master[185];
    char line[511], *a;
    struct hostent *dest;

    if (argc != 3) {
	fprintf(stderr, "usage: %s <sec> <usec>\n", *argv);
	exit(1);
    }
    
    printf("\n----->sending<-----\n");

    cnt = 0;
    while (scanf("%s", line) != EOF && cnt != 185) {
	printf("%s\n", line);
	if ((dest = gethostbyname(line)) != NULL)
	    bcopy(dest->h_addr_list[0], &iaddr, 4);
	else if ((iaddr = inet_addr(line)) == -1)
	    continue;
	sonar_master[cnt++].host = iaddr;
    }

    bcopy((char *)sonar_master, (char *)sonar, sizeof(sonar_master));
    printf("----->receiving<-----\n");
    i = sonar_probe(sonar, &cnt, atoi(argv[1]), atoi(argv[2]));
    if (i != SONAR_SUCCESS) {
	switch (i) {
	  case SONAR_ERROR:
	    a = "SONAR_ERROR";
	    break;
	  case SONAR_TIMEDOUT:
	    a = "SONAR_TIMEDOUT";
	    break;
	  case SONAR_TOOBUSY:
	    a = "SONAR_TOOBUSY";
	    break;
	  case SONAR_NOSERVER:
	    a = "SONAR_NOSERVER";
	    break;
	  case SONAR_BADVER:
	    a = "SONAR_BADVER";
	}
	printf("*** sonar_probe failed.... %s\n\n", a);
	exit(1);
    }

    for (i=0; i < cnt; i++)
	printf("host = %s\tmetric = %u\n",
	       inet_ntoa(*((struct in_addr *)&sonar[i].host)),
	       sonar[i].metric);

    printf("\n");
    exit(0);
}
