#include "findnets.h"

/*
 * findbroadcast
 *
 * Description: This function finds all available AF_INET broadcast addresses
 *              and returns an int array containing the addresses.  The array
 *              is NULL terminated.
 *
 */
int *
findbroadcast()
{
    char buf[BUFSIZ];
    struct ifreq *ifrp;
    struct ifconf ifc;
    struct sockaddr_in *sin;
    int s, n, j, flags;
    int *array, i=0, dup;

    
    array = (int *)malloc(sizeof(int));
    array[0] = 0;
    
    s = socket(AF_INET, SOCK_DGRAM, 0);
    if (s < 0) {
	perror("socket");
	return array;
    }

    ifc.ifc_len = sizeof(buf);
    ifc.ifc_buf = buf;
    if (ioctl(s, SIOCGIFCONF, (char *)&ifc) < 0) {
	perror("ioctl: (SIOCGIFCONF)");
	close(s);
	return array;
    }

    ifrp = ifc.ifc_req;
    for (n = ifc.ifc_len / sizeof(struct ifreq); n > 0; n--, ifrp++) {
	dup = 0;
	close(s);
	s = socket(AF_INET, SOCK_DGRAM, 0);
	if (s < 0) {
	    perror("socket");
	    return array;
	}

	if (ioctl(s, SIOCGIFFLAGS, ifrp) < 0) {
	    perror("ioctl (SIOCGIFFLAGS)");
	    return array;
	}
	flags = ifrp->ifr_flags;

	if (flags & IFF_BROADCAST) {
	    if (ioctl(s, SIOCGIFBRDADDR, ifrp) < 0) {
		if (errno == EADDRNOTAVAIL)
		    continue;
		perror("ioctl: (SIOCGIFBRDADDR)");
		return array;
	    }
	    
	    sin = (struct sockaddr_in *)&ifrp->ifr_addr;
	    for (j=0; array[j] != 0; j++)
		if (array[j] == sin->sin_addr.s_addr)
		    dup = 1;
	
	    if (!dup&& sin->sin_family == AF_INET && sin->sin_addr.s_addr !=0){
		i++;
		array = (int *) realloc(array, sizeof(int)*(i+1));
		array[i-1] = sin->sin_addr.s_addr;
		array[i] = 0;
	    }
	}
    }
    close(s);
    return array;
}


/*
 * findnets
 *
 * Description: This function finds all available AF_INET addresses and
 *              their netmasks.  It returns an array of struct addrlist's
 *              containing the addresses.  The last addrlist.addr is set
 *              to 0.
 *
 */
Addrlist
findnets()
{
    char buf[BUFSIZ];
    struct ifreq *ifrp;
    struct ifconf ifc;
    struct sockaddr_in *sin;
    int s, n, j, flags;
    int i=0, dup;
    Addrlist array;

    
    array = (Addrlist)malloc(sizeof(struct addrlist));
    array[0].addr = 0;
    
    s = socket(AF_INET, SOCK_DGRAM, 0);
    if (s < 0) {
	perror("socket");
	return array;
    }

    ifc.ifc_len = sizeof(buf);
    ifc.ifc_buf = buf;
    if (ioctl(s, SIOCGIFCONF, (char *)&ifc) < 0) {
	perror("ioctl: (SIOCGIFCONF)");
	close(s);
	return array;
    }
    
    ifrp = ifc.ifc_req;
    for (n = ifc.ifc_len / sizeof(struct ifreq); n > 0; n--, ifrp++) {
	dup = 0;
	close(s);
	s = socket(AF_INET, SOCK_DGRAM, 0);
	if (s < 0) {
	    perror("socket");
	    return array;
	}
	
	if (ioctl(s, SIOCGIFFLAGS, ifrp) < 0) {
	    perror("ioctl (SIOCGIFFLAGS)");
	    return array;
	}
	flags = ifrp->ifr_flags;
	
	if (!(flags & IFF_LOOPBACK)) {
	    if (ioctl(s, SIOCGIFADDR, ifrp) < 0) {
		continue;
	    }
	    
	    sin = (struct sockaddr_in *)&ifrp->ifr_addr;
	    for (j=0; array[j].addr != 0; j++)
		if (array[j].addr == sin->sin_addr.s_addr)
		    dup = 1;

	    if (!dup&& sin->sin_family == AF_INET && sin->sin_addr.s_addr !=0){
		i++;
		array = (Addrlist)realloc(array,sizeof(struct addrlist)*(i+1));
		array[i-1].addr = sin->sin_addr.s_addr;
		if (ioctl(s, SIOCGIFNETMASK, ifrp) < 0)
		    perror("ioctl (SIOCGIFNETMASK)");
		sin = (struct sockaddr_in *)&ifrp->ifr_addr;
		array[i-1].mask = sin->sin_addr.s_addr;
		array[i].addr = 0;
	    }
	}
    }
    close(s);
    return array;
}
