#include "lruhash.h"

#define TRUE  1
#define FALSE 0


/*
 * lruhash_create
 *
 * Description: Create an lruhash and returns a pointer to it.
 *
 * hashsize is the size of the chained hash table.
 *
 * numnodes is the maximum number of entries into the hash table before
 * using the least recently used node
 *
 */
Hash
hash_create(hashsize, numnodes)
int hashsize,
    numnodes;
{
    Hash x;
    Hash_node node;
    int i;
    
    x = (Hash) malloc(sizeof(struct hash));
    if (!x) {
	fprintf(stderr,"out of memory!\n");
	exit(1);
    }

    x->hash = (Hash_node) malloc(sizeof(struct hash_node) * hashsize);
    if (!x->hash) {
	fprintf(stderr,"out of memory!\n");
	exit(1);
    }

    /* initialize hash table */
    for (i=0; i < hashsize; i++) {
	node = x->hash + i;
	node->hash_next = node->hash_prev = node;
    }
    
    x->lru_chain = (Hash_node) malloc(sizeof(struct hash_node) * (numnodes + 2));
    if (!x->lru_chain) {
	fprintf(stderr,"out of memory!\n");
	exit(1);
    }

    x->free = x->lru_chain + 1;

    x->size = hashsize;
    x->free_size = numnodes;

    node = x->lru_chain;
    node->lru_next = node->lru_prev = node->hash_next = node->hash_prev = node;

    return x;
}



void
hash_insert(hash, hashval, host)
Hash hash;
unsigned int hashval;
Hash_node host;
{
    Hash_node x, y;

    x = hash_find(hash, hashval, host->host);

    if (x) {
	/* delete from lru chain */
	x->lru_next->lru_prev = x->lru_prev;
	x->lru_prev->lru_next = x->lru_next;

	/* delete from hash */
	x->hash_next->hash_prev = x->hash_prev;
	x->hash_prev->hash_next = x->hash_next;

	/* Place this node at the end of the lru_chain for reuse */
	hash_free(hash, x);
    }
    
    /* insert into lru chain */
    y = hash->lru_chain;
    host->lru_prev = y;
    host->lru_next = y->lru_next;

    y->lru_next->lru_prev = host;
    y->lru_next = host;

    /* insert into chained hash */
    y = hash->hash + (hashval % hash->size);
    host->hash_prev = y;
    host->hash_next = y->hash_next;

    y->hash_next->hash_prev = host;
    y->hash_next = host;
}


/*
 * hash_find
 *
 * Description: Returns the node matching the key.  If the key was not found
 *              it returns NULL.
 *
 */
Hash_node
hash_find(hash, hashval, host)
Hash hash;
unsigned int hashval;
unsigned int host;
{
    Hash_node x, y;

    x = hash->hash + (hashval % hash->size);
    y = x->hash_next;

#ifdef _DEBUG_
/*    fprintf(stderr, "*** HASH VALUE = %d\n", hashval % hash->size);*/
#endif
    
    while (y != x && y->host != host)
	y = y->hash_next;

    if (y == x || y->host != host)
	return NULL;

    /* update lru chain */
    y->lru_next->lru_prev = y->lru_prev;
    y->lru_prev->lru_next = y->lru_next;

    y->lru_next = hash->lru_chain->lru_next;
    y->lru_prev = hash->lru_chain;

    hash->lru_chain->lru_next->lru_prev = y;
    hash->lru_chain->lru_next = y;


    return y;
}


/*
 * hash_getnode
 *
 * Description: Returns a free hash node that it gets from the internal
 *              freelist or the least recently used node in the hash table.
 *
 */
Hash_node
hash_getnode(hash)
Hash hash;
{
    Hash_node x;

    if (hash->free_size) {
	/* Use node from free list */
	x = hash->free;
	hash->free = x + 1;
	hash->free_size--;
    }
    else {
	/* get least recently used node */
	x = hash->lru_chain->lru_prev;
	
	/* delete from lru chain */
	x->lru_next->lru_prev = x->lru_prev;
	x->lru_prev->lru_next = x->lru_next;

	/* delete from hash */
	x->hash_next->hash_prev = x->hash_prev;
	x->hash_prev->hash_next = x->hash_next;

	while (x->clients_waiting)
	    client_update_delete(&(x->clients_waiting),
				 x->clients_waiting->client);
    }

    return x;
}


/*
 * hash_free
 *
 * Description: Puts a hash node onto the internal freelist
 *
 */
void
hash_free(hash, node)
Hash hash;
Hash_node node;
{
    /* Stick it at the end of the least recently used chain */
    node->lru_next = hash->lru_chain;
    node->lru_prev = hash->lru_chain->lru_prev;
    node->lru_prev->lru_next = node;
    hash->lru_chain->lru_prev = node;
}


/*
 * hash_index
 *
 * Description: A hash function.  Maps Internet addresses to a hash bucket.
 */
unsigned int
hash_index(host) 
unsigned int host;
{ 
    return host;
}
