#include "recv_udp.h"



/*
 * recv_udp
 *
 * Description: Gets a packet from the socket and checks to see if it's
 *              a valid request.  If so, process the request.  If not,
 *              either drop it or send back an error.
 *
 */
void
recv_udp(s) 
int s;
{ 
   char buf[1536];
   int tmp, r_ln, version;
   struct sockaddr_in reply_addr;
   extern Masklist query_mask;
   extern int *broadcast_addrs;
   Masklist mlptr;
   
   r_ln=sizeof(reply_addr);
   
   if ((tmp=recvfrom(s,buf,1536,0, (struct sockaddr *)&reply_addr, &r_ln ))<0){
       if (tmp == EWOULDBLOCK) {
	   return;
       }
       perror("recvfrom error");
       return;
   }

#ifdef _DEBUG_
   fprintf(stderr,"received request from %s\n",inet_ntoa(reply_addr.sin_addr));
#endif

   for (tmp=0; broadcast_addrs[tmp] != NULL; tmp++) {
#ifdef _DEBUG_
       fprintf(stderr, "broadcast[%d] = %s\n", tmp, inet_ntoa(*((struct in_addr *)&broadcast_addrs[tmp])));
#endif
       if (reply_addr.sin_addr.s_addr == broadcast_addrs[tmp]) {
#ifdef _DEBUG_
	   fprintf(stderr, "dropping packet from broadcast address\n");
#endif
	   return;
       }
   }
   
   tmp = 1;
   ml_traverse(mlptr, query_mask) {
#ifdef _DEBUG_
       fprintf(stderr, "mask %s  %x\n", inet_ntoa(*((struct in_addr *)&mlptr->addr)), mlptr->mask);
#endif
       if ((reply_addr.sin_addr.s_addr & mlptr->mask) == (mlptr->addr &
	   mlptr->mask)){
#ifdef _DEBUG_
	   fprintf(stderr, "request allowed\n");
#endif
	   tmp = 0;
	   break;
       }
   }   
   
   if (tmp && query_mask->next != query_mask) {
#ifdef _DEBUG_
       fprintf(stderr, "request denied\n");
#endif
       return;
   }
   
   version = ntohl(*((int *)buf));

   if (version > SONAR_VERSION || SONAR_LOW_VERSION > version)
       send_error(buf,&reply_addr,s, VERSION_MISMATCH);
   else
       decode_udp(buf,&reply_addr, s, r_ln);
} 



/*
 * send_error
 *
 * Description: Makes an error reply and sends it off.
 *
 */
void
send_error(buf, reply_addr, s, error)
char *buf;
struct sockaddr_in *reply_addr;
int s,
    error;
{
    int i;
    char *ptr;

    *((int *)buf) = htonl(SONAR_VERSION);
    ptr = buf + 4;
    
    error = htonl(error);
    bcopy(&error, ptr, 4);
    ptr += 8;

    i = htonl(SONAR_LOW_VERSION);
    bcopy(&i, ptr, 4);
    ptr += 4;

    i = htonl(SONAR_VERSION);
    bcopy(&i, ptr, 4);

    if (sendto(s, buf, 16, 0, (struct sockaddr *)reply_addr, 16) != 16)
	perror("sendto");
	
    return;
}
